@@ -25,8 +25,8 @@ import (
 	"github.com/containernetworking/cni/libcni"
 	"github.com/golang/glog"
 	"github.com/intel/multus-cni/types"
+	netv1 "github.com/k8snetworkplumbingwg/network-attachment-definition-client/pkg/apis/k8s.cni.cncf.io/v1"
 	"github.com/pkg/errors"
-
 	"k8s.io/api/admission/v1beta1"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -52,9 +52,51 @@ var (
 	clientset kubernetes.Interface
 )
 
-func validateNetworkAttachmentDefinition(netAttachDef types.NetworkAttachmentDefinition) (bool, error) {
+// validateCNIConfig verifies following fields
+// conf: 'type'
+// conflist: 'plugins' and 'type'
+func validateCNIConfig(config []byte) error {
+	var c map[string]interface{}
+	if err := json.Unmarshal(config, &c); err != nil {
+		return err
+	}
+
+	// Identify target is single CNI config or plugins
+	if p, ok := c["plugins"]; ok {
+		// CNI conflist
+		// check 'type' field for each plugin in 'plugins'
+		plugins := p.([]interface{})
+		for _, v := range plugins {
+			plugin := v.(map[string]interface{})
+			if _, ok := plugin["type"]; !ok {
+				return fmt.Errorf("missing 'type' in plugins")
+			}
+		}
+	} else {
+		// single CNI config
+		if _, ok := c["type"]; !ok {
+			return fmt.Errorf("missing 'type' in cni config")
+		}
+	}
+	return nil
+}
+
+// preprocessCNIConfig process CNI config bytes as following (that multus does too)
+// - if 'name' is missing, 'name' is filled
+func preprocessCNIConfig(name string, config []byte) ([]byte, error) {
+	var c map[string]interface{}
+	if err := json.Unmarshal(config, &c); err != nil {
+		if n, ok := c["name"]; !ok || n == "" {
+			c["name"] = name
+		}
+	}
+	configBytes, err := json.Marshal(c)
+	return configBytes, err
+}
+
+func validateNetworkAttachmentDefinition(netAttachDef netv1.NetworkAttachmentDefinition) (bool, error) {
 	nameRegex := `^[a-z-1-9]([-a-z0-9]*[a-z0-9])?$`
-	isNameCorrect, err := regexp.MatchString(nameRegex, netAttachDef.Metadata.Name)
+	isNameCorrect, err := regexp.MatchString(nameRegex, netAttachDef.GetName())
 	if !isNameCorrect {
 		err := errors.New("net-attach-def name is invalid")
 		glog.Info(err)
@@ -70,11 +112,18 @@ func validateNetworkAttachmentDefinition(netAttachDef types.NetworkAttachmentDef
 
 	var confBytes []byte
 	if netAttachDef.Spec.Config != "" {
-
 		// try to unmarshal config into NetworkConfig or NetworkConfigList
 		//  using actual code from libcni - if succesful, it means that the config
 		//  will be accepted by CNI itself as well
-		confBytes = []byte(netAttachDef.Spec.Config)
+		confBytes, err = preprocessCNIConfig(netAttachDef.GetName(), []byte(netAttachDef.Spec.Config))
+		if err != nil {
+			err := errors.Wrap(err, "invalid json")
+			return false, err
+		}
+		if err := validateCNIConfig(confBytes); err != nil {
+			err := errors.Wrap(err, "invalid config")
+			return false, err
+		}
 		_, err = libcni.ConfListFromBytes(confBytes)
 		if err != nil {
 			glog.Infof("spec is not a valid network config list: %s - trying to parse into standalone config", err)
@@ -286,9 +335,9 @@ func parsePodNetworkObjectName(podnetwork string) (string, string, string, error
 	return netNsName, networkName, netIfName, nil
 }
 
-func deserializeNetworkAttachmentDefinition(ar *v1beta1.AdmissionReview) (types.NetworkAttachmentDefinition, error) {
+func deserializeNetworkAttachmentDefinition(ar *v1beta1.AdmissionReview) (netv1.NetworkAttachmentDefinition, error) {
 	/* unmarshal NetworkAttachmentDefinition from AdmissionReview request */
-	netAttachDef := types.NetworkAttachmentDefinition{}
+	netAttachDef := netv1.NetworkAttachmentDefinition{}
 	err := json.Unmarshal(ar.Request.Object.Raw, &netAttachDef)
 	return netAttachDef, err
 }
